@@ -1,49 +1,43 @@
+import os
 import sys
 from sys import stderr, stdout
-
+import colorama
 import covert
 from covert.cli import main_benchmark, main_dec, main_enc
 
+hdrhelp = """\
+Usage:
+  covert enc [files] [recipients] [signatures] [-A | -o unsuspicious.dat [-a]]
+  covert dec [-A | unsuspicious.dat] [-i id_ed25519] [-o filesfolder]
+  covert benchmark
+
+Note: covert enc/dec without arguments ask for password and message. Files and
+folders get attached together with a message if 'enc -' is specified.
+"""
+
 enchelp = """\
-Encryption options:
-  -p, --passphrase  Ask for passphrase (default if no other recipients)
-  -r --recipient K  Encrypt to the specified public key: RW…, AAAA…, age1…
-  -R --keyfile FN   Read recipients from a local keyfile, or:
-                      github:foo - Download foo's SSH keys from Github
-  --wide-open       No authentication required, anyone can open the file
-
-  -i --identity FN  Sign the message using secret keys (file or key)
-  -o --output FN    Encrypted file to output (binary unless -a is used)
-  -a --armor        Encode to Base64 format (default if output is console)
-  --pad PERCENT     Random padding preferred amount (0 disable, 5 default)
-
-Filenames or folders given are added to the archive, omitting full paths.
-A message with file attachments can be written by adding a hyphen '-'.
+  -p                Passphrase recipient (default)
+  --wide-open       Anyone can open the file (no recipients)
+  -r PKEY -R FILE   Recipient pubkey, .pub file or github:username
+  -i SKEY           Sign with a secret key (string token or id file)
+  -A                Auto copy&paste: ciphertext is copied
+  -o FILENAME       Encrypted file to output (binary unless -a is used)
+  --pad PERCENT     Preferred padding amount (default 5 %)
 """
 
 dechelp = """\
-Decryption options:
-  -o --output PATH  A folder/ to extract files or a file for message output
-  -i --identity FN  Use the given keyfile or secret key instead of password
+  -A                Auto copy&paste: ciphertext is pasted
+  -i SKEY           Decrypt with secret key (token or file)
+  -o FILEFOLDER     Extract any attached files to
 """
 
 cmdhelp = f"""\
 Covert {covert.__version__} - A file and message encryptor with strong anonymity
  💣  Things encrypted with this developer preview mayn't be readable evermore
 
-Usage:
-  covert enc [-p | -r PUBKEY | -R KEYFILE]... [-o OUTPUT] [FILE | DIR]...
-  covert dec [-i PRIVKEY]... [INPUT] [-o OUTPUT]
-  covert benchmark
-
-Note: covert enc/dec with no other arguments will use the console.
-
+{hdrhelp}
 {enchelp}
 {dechelp}
-Example:
-  $ minisign -G -p foo.pub -s foo.key
-  $ covert enc -R foo.pub - pictures/ -o unsuspicious.dat
-  $ covert dec -i foo.key unsuspicious.dat -o new_pictures/
 """
 
 
@@ -60,6 +54,7 @@ class Args:
     self.identities = []
     self.padding = "5"
     self.armor = None
+    self.paste = None
     self.debug = None
 
 
@@ -72,13 +67,17 @@ encargs = dict(
   identities='-i --identity'.split(),
   outfile='-o --out --output'.split(),
   armor='-a --armor'.split(),
+  paste='-A'.split(),
   padding='--pad --padding'.split(),
   debug='--debug'.split(),
 )
 
 decargs = dict(
+  askpass='-p --passphrase'.split(),
+  passwords='--password'.split(),
   identities='-i --identity'.split(),
   outfile='-o --out --output'.split(),
+  paste='-A'.split(),
   debug='--debug'.split(),
 )
 
@@ -100,22 +99,26 @@ def argparse():
     sys.exit(0)
   ad = {}
   args = Args()
+  # Separate mode selector from other arguments
+  if av[0].startswith("-") and len(av[0]) > 2:
+    av.insert(1, f'-{av[0][2:]}')
+    av[0] = av[0][:2]
   # Support a few other forms for Age etc. compatibility (but only as the first arg)
-  if av[0].lower() in ('enc', 'encrypt', '-e', '--encrypt'):
-    args.mode, ad, modehelp = 'enc', encargs, enchelp
-  elif av[0].lower() in ('dec', 'decrypt', '-d', '--decrypt'):
-    args.mode, ad, modehelp = 'dec', decargs, dechelp
-  elif av[0].lower() in ('bench', 'benchmark'):
-    args.mode, ad, modehelp = 'benchmark', benchargs, "Benchmark mode takes no arguments\n"
-  elif next((k for k, v in encargs.items() if av[0] in v), None) is not None:
-    args.mode, ad, modehelp = 'enc', encargs, enchelp
-    av.insert(0, None)
+  if av[0] in ('enc', 'encrypt', '-e'):
+    args.mode, ad, modehelp = 'enc', encargs, f"{hdrhelp}\nEncryption options:\n{enchelp}"
+  elif av[0] in ('dec', 'decrypt', '-d'):
+    args.mode, ad, modehelp = 'dec', decargs, f"{hdrhelp}\nEncryption options:\n{enchelp}"
+  elif av[0] in ('bench', 'benchmark'):
+    args.mode, ad, modehelp = 'benchmark', benchargs, f"{hdrhelp}"
   else:
     stderr.write(' 💣  Invalid or missing command (enc/dec/benchmark).\n')
     sys.exit(1)
-
+  
   aiter = iter(av[1:])
+  longargs = [flag[1:] for switches in ad.values() for flag in switches if flag.startswith("--")]
+  shortargs = [flag[1:] for switches in ad.values() for flag in switches if not flag.startswith("--")]
   for a in aiter:
+    aprint = a
     if not a.startswith('-'):
       args.files.append(a)
       continue
@@ -125,33 +128,43 @@ def argparse():
     if a == '--':
       args.files += args
       break
-    if not a.startswith('--') and len(a) > 2:
-      stderr.write(f' 💣  Short arguments cannot be combined: covert {args.mode} {a}\n')
-      sys.exit(1)
     if a.startswith('--'):
       a = a.lower()
-    argvar = next((k for k, v in ad.items() if a in v), None)
-    if argvar is None:
-      stderr.write(f'{modehelp}\n 💣  Unknown argument: covert {args.mode} {a}\n')
-      sys.exit(1)
-    try:
-      var = getattr(args, argvar)
-      if isinstance(var, list):
-        var.append(next(aiter))
-      elif isinstance(var, str):
-        setattr(args, argvar, next(aiter))
-      elif isinstance(var, int):
-        setattr(args, argvar, var + 1)
-      else:
-        setattr(args, argvar, True)
-    except StopIteration:
-      stderr.write(f'{modehelp}\n 💣  Argument parameter missing: covert {args.mode} {a} …\n')
-      sys.exit(1)
+    if not a.startswith('--') and len(a) > 2:
+      if any(arg not in shortargs for arg in list(a[1:])):
+        falseargs = [arg for arg in list(a[1:]) if arg not in shortargs]
+        stderr.write(f' 💣  {falseargs} is not an argument: covert {args.mode} {a}\n')
+        sys.exit(1)
+      a = [f'-{shortarg}' for shortarg in list(a[1:]) if shortarg in shortargs]
+    if isinstance(a, str):
+      a = [a]
+    for i, av in enumerate(a):
+      argvar = next((k for k, v in ad.items() if av in v), None)
+      if isinstance(av, int):
+        continue
+      if argvar is None:
+        stderr.write(f'{modehelp}\n 💣  Unknown argument: covert {args.mode} {aprint}\n')
+        sys.exit(1)
+      try:
+        var = getattr(args, argvar)
+        if isinstance(var, list):
+          var.append(next(aiter))
+        elif isinstance(var, str):
+          setattr(args, argvar, next(aiter))
+        elif isinstance(var, int):
+          setattr(args, argvar, var + 1)
+        else:
+          setattr(args, argvar, True)
+      except StopIteration:
+        stderr.write(f'{modehelp}\n 💣  Argument parameter missing: covert {args.mode} {aprint} …\n')
+        sys.exit(1)
 
   return args
 
 
 def main():
+  colorama.init()
+  # CLI argument processing
   args = argparse()
   if len(args.outfile) > 1:
     raise ValueError('Only one output file may be specified')
