@@ -1,7 +1,64 @@
 import io
 import os
+import sys
 import time
 from contextlib import contextmanager
+from shutil import get_terminal_size
+
+
+def editor():
+  with fullscreen() as term:
+    term.write(f'\x1B[1;1H\x1B[1;44m   〰 ENTER MESSAGE 〰   (ESC to finish)\x1B[0K\x1B[0m\n')
+    data = ""
+    startrow = row = col = 0
+    while True:
+      for key in term.reader():
+        lines = data.split("\n") + [""]
+        if key == 'ESC':
+          return data
+        elif key == 'BACKSPACE':
+          if col > 0:
+            col -= 1
+            lines[row] = lines[row][:col] + lines[row][col + 1:]
+          elif row > 0:
+            row -= 1
+            col = len(lines[row])
+            lines[row] += lines[row + 1]
+            del lines[row + 1]
+        elif key == 'LEFT':
+          if col > 0:
+            col -= 1
+          elif row > 0:
+            row -= 1
+            col = len(lines[row])
+        elif key == 'RIGHT':
+          if col < len(lines[row]):
+            col += 1
+          elif row < len(lines) - 1:
+            row += 1
+            col = 0
+        elif key == 'UP' and row > 0:
+          row -= 1
+          col = min(col, len(lines[row]))
+        elif key == 'DOWN' and row < len(lines) - 1:
+          row += 1
+          col = min(col, len(lines[row]))
+        elif key == 'ENTER':
+          lines.insert(row + 1, lines[row][col:])
+          lines[row] = lines[row][:col]
+          col = 0
+          row += 1
+        elif len(key) == 1:
+          lines[row] = lines[row][:col] + key + lines[row][col:]
+          col += 1
+        if not lines[-1]:
+          del lines[-1]
+        data = "\n".join(lines)
+      win = get_terminal_size()
+      startrow = min(max(0, row - 1), startrow)
+      startrow = max(row - win.lines + 2, startrow)
+      draw = "\x1B[0K\n".join(l[:win.columns - 1] for l in data.split("\n")[startrow:startrow + win.lines - 1])
+      term.write(f"\x1B[2;1H{draw}\x1B[0J\x1B[{row - startrow + 2};{col+1}H")
 
 
 def read_hidden(prompt):
@@ -24,10 +81,11 @@ def read_hidden(prompt):
           elif len(key) == 1:
             data += key
           t = time.monotonic()
-        term.write(f"\x1B7  ({len(data)}) \x1B8")
+        status = f"  ({len(data)}) "
+        term.write(f"{status}\x1B[{len(status)}D")
     finally:
       # Return to start of line and clear the prompt
-      term.write(f"\x1B[0m\r\x1B[K")
+      term.write(f"\x1B[0m\r\x1B[0K")
 
 
 @contextmanager
@@ -76,11 +134,11 @@ except (ImportError, AttributeError):
 
 @contextmanager
 def modeswitch(term):
-  term.write('\x1B[?1049h')
+  term.write('\x1B[?1049h\x1B[2J')
   try:
     yield
   finally:
-    term.write('\x1B[?1049l')
+    term.write('\x1B[2J\x1B[?1049l')
 
 
 @contextmanager
@@ -102,13 +160,14 @@ class Terminal:
       self.tty.flush()
     else:
       text = text.replace('\n', '\r\n')
-      for ch in text:
-        msvcrt.putwch(ch)
+      sys.stderr.write(text)
+      #for ch in text:
+      #  msvcrt.putwch(ch)
 
   def reader_windows(self):
     while True:
       ch = msvcrt.getwch()
-      if ch == '\x00':
+      if ch == '\x00' or ch == 'à':
         ch = msvcrt.getwch()
         if ch == 'H': yield 'UP'
         elif ch == 'P': yield 'DOWN'
