@@ -4,6 +4,8 @@ from secrets import token_bytes
 import nacl.bindings as sodium
 
 from covert import pubkey, sign
+import pytest
+
 
 # Test vectors from https://age-encryption.org/v1
 AGE_PK = "age1zvkyg2lqzraa2lnjvqej32nkuu0ues2s82hzrye869xeexvn73equnujwj"
@@ -35,12 +37,25 @@ def test_age_key_decoding_and_encoding():
 
 
 def test_ssh_key_decoding():
-  pk, = pubkey.read_pk_file("keys/ssh_ed25519.pub")
-  sk, = pubkey.read_sk_file("keys/ssh_ed25519")
+  pk, = pubkey.read_pk_file("tests/keys/ssh_ed25519.pub")
+  sk, = pubkey.read_sk_file("tests/keys/ssh_ed25519")
   assert pk.comment == "test-key@covert"
+  assert sk.comment == "test-key@covert"
   assert pk == sk
 
 
+def test_ssh_pw_keyfile(mocker):
+  mocker.patch('covert.passphrase.ask', return_value=(b"password", True))
+  sk, = pubkey.read_sk_file("tests/keys/ssh_ed25519_password")
+  assert sk.comment == "password-key@covert"
+
+
+def test_ssh_wrong_password(mocker):
+  mocker.patch('covert.passphrase.ask', return_value=(b"not this password", True))
+  with pytest.raises(ValueError):
+    sk, = pubkey.read_sk_file("tests/keys/ssh_ed25519_password")
+
+
 def test_key_exchange():
   # Alice sends a message to Bob
   nonce = token_bytes(12)
@@ -87,11 +102,10 @@ def test_signing():
 
 def test_signature_high_level():
   for i in range(100):
-    presk = token_bytes(32)
-    pk, sk = sodium.crypto_box_seed_keypair(presk)
-    sk = sk[:32]
+    key = pubkey.Key()
+    pub = pubkey.Key(edpk=key.edpk)
     blkhash = token_bytes(64)
     # Sign & verify
-    signature = sign.signature(presk, blkhash)
-    msg = sign.verify(pk, blkhash, signature)
+    signature = sign.signature(key, blkhash)
+    msg = sign.verify(pub, blkhash, signature)
     assert msg == blkhash
